/*
 * Hqp.h -- some declarations for huge quadratic programming
 *          (Meschach declarations and some extensions)
 *
 * rf, 5/28/94
 *
 * rf, 2/12/97
 *   new result Hqp_Degenerate for singular matrix errors
 *   no result Hqp_Insolvable anymore as this can't be decited on QP-level
 */

/*
    Copyright (C) 1994--1998  Ruediger Franke

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; 
    version 2 of the License.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (file COPYING.LIB);
    if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef Hqp_H
#define Hqp_H

#include <math.h>
#include "Meschach.h"

/** return HQP version string */
extern const char *hqp_version();

/** possible results of QP solver */
enum Hqp_Result {
  Hqp_Optimal = 0,
  Hqp_Feasible,
  Hqp_Infeasible,
  Hqp_Suboptimal,
  Hqp_Degenerate
};

/** array of result strings for Hqp_Result */
extern char *hqp_result_strings[];

/** Infinity for non existing constraints and numerical overflow */
const double Inf = HUGE_VAL;

/** check if a number is finite */
inline bool is_finite(double x)
{
  return -Inf < x && x < Inf;
}

#endif
