/*
 * Oc_Euler.C -- class definition
 *
 * rf, 6/24/94
 */

/*
    Copyright (C) 1994--1998  Ruediger Franke

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; 
    version 2 of the License.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (file COPYING.LIB);
    if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <assert.h>

#include "Oc_Euler.h"


//--------------------------------------------------------------------------
void Oc_Euler::val(const VEC *x, const VEC *u,
		   const Oc_Horizon *hor, int k, ValDeqProc *val_deq,
		   VEC *f)
{
  assert(f && f->dim == x->dim);

  Real dt = hor->dt();

  (*val_deq)(x, u, (Real)k*dt, f);
  sv_mlt(dt, f, f);
  v_add(x, f, f);
}

//--------------------------------------------------------------------------
void Oc_Euler::grd(const VEC *x, const VEC *u,
		   const Oc_Horizon *hor, int k, GrdDeqProc *grd_deq,
		   MAT *fx, MAT *fu)
{
  assert(fx && fx->m == x->dim && fx->n == x->dim);
  assert(fu && fu->m == x->dim && fx->n == u->dim);

  int	i, n = x->dim;
  Real dt = hor->dt();

  (*grd_deq)(x, u, (Real)k*dt, fx, fu);

  sm_mlt(dt, fx, fx);
  for(i=0; i<n; i++)
    fx->me[i][i] += 1.0;
  
  sm_mlt(dt, fu, fu);
}


//==========================================================================
