#
# some Tcl procedures for the Hqp package
#
# rf, 9/8/96
#
# rf, 4/6/00
#  - added hqp_solve_hot for hot start of SQP solver
#

proc hqp_exit {reason} {
  exit 0
}

proc hqp_solve_hot {{stream stdout}} {
  hqp_solve $stream 1
}

proc hqp_solve {{stream stdout} {hot 0}} {

  global qp_iters

  if {![info exists qp_iters] || [sqp_iter] == 0} {
    set qp_iters 0
  }
  set nullsteps 0

  if {[sqp_iter] == 0 || $hot} {
    puts $stream [format "%3s %12s %10s %10s \[%3s %3.3s\] %10s %10s %8s"\
                  it obj ||inf|| ||grdL||\
                  qp result ||s|| s'Qs stepsize]
  }

  while 1 {

    if {$qp_iters == 0} {
      if {!$hot} {
	# can't re-use higher order information
	sqp_qp_update
      }
      puts -nonewline $stream [format "%3d %12.6g %10.4g %10.4g " \
         [sqp_iter] [prg_f] [sqp_norm_inf] [sqp_norm_grd_L]]
      flush $stream
    } else {
      puts -nonewline $stream [format "%3d %12.6g %10.4g " \
         [sqp_iter] [prg_f] [sqp_norm_inf]]
      flush $stream

      # additional break test as norm_inf changed with taken step
      # (Don't use this brake test after cold start,
      #  as Hessian approximation is not updated anymore!
      #  This update is useful for subsequent hot starts.)
      if {$hot && [sqp_iter] > 0 && [sqp_sQs] >= 0.0 && !$hela_restart} {
        if {[sqp_norm_inf] < [sqp_eps] && [qp_result] == "optimal"} {
          if {[sqp_sQs] < [expr [sqp_eps]*[sqp_eps]]} break
 	  if {[sqp_iter] > 2} {
            if {[sqp_norm_s] < [expr [sqp_eps]*[sqp_norm_x]]
 	        && [sqp_norm_df] < [expr [sqp_eps]*abs([prg_f])]
	        && [sqp_sQs] < [sqp_eps]} break
  	  }
        }
      }

      sqp_qp_update

      puts -nonewline $stream [format "%10.4g " [sqp_norm_grd_L]]
      flush $stream
    }

    # test for failed hot start
    if {0 && $hot} {
      if {$qp_iters == 0} {
        set norm_grd_L0 [sqp_norm_grd_L]
      } elseif {[sqp_norm_grd_L] > 1.0 &&
	        [expr [sqp_eps]*[sqp_norm_grd_L]] >= $norm_grd_L0} {
        puts "\nRestart cold:"
        prg_setup
        prg_simulate
        sqp_init
        return [hqp_solve $stream]
      }
    }
    
    # test for bad Hessian
    if {[sqp_xQx] < 0.0} {
      sqp_hela_restart
      set hela_restart 1
    } else {
      set hela_restart 0
    }
    set xQx_old [sqp_xQx]

    if {[sqp_iter] > 0} {
      if {[sqp_norm_inf] < [sqp_eps]} {
        if {[sqp_norm_grd_L] < [sqp_eps]} break
      }
    }

    sqp_qp_solve
    incr qp_iters [qp_iter]

    puts -nonewline $stream [format "\[%3d %3.3s\] " \
	[qp_iter] [qp_result]]
    flush $stream

    if {[qp_iter] == 0} {
	puts $stream ""
	error [qp_result]
    }

    puts -nonewline $stream [format "%10.4g %10.4g " \
        [sqp_norm_s] [sqp_sQs]]

    if {[sqp_sQs] < 0.0} {
      sqp_hela_restart
    }

    if {[sqp_iter] > 0 && [sqp_sQs] >= 0.0 && !$hela_restart} {
      if {[sqp_norm_inf] < [sqp_eps] && [qp_result] == "optimal"} {
        if {[sqp_sQs] < [expr [sqp_eps]*[sqp_eps]]} break
 	if {[sqp_iter] > 2} {
          if {[sqp_norm_s] < [expr [sqp_eps]*[sqp_norm_x]]
 	      && [sqp_norm_df] < [expr [sqp_eps]*abs([prg_f])]
	      && [sqp_sQs] < [sqp_eps]} break
	}
      }
    }

    sqp_step

    puts $stream [format "%8.3g" [sqp_alpha]]

    if {[qp_iter] >= [qp_max_iters]} {
        error subiters
    }
    if {[sqp_iter] >= [sqp_max_iters]} {
        error iters
    }
    if {[sqp_inf_iters] >= [sqp_max_inf_iters]} {
	if {[qp_result] == "suboptimal"} {
	    error infeasible
	} else {
	    error degenerate
	}
    }

    if {[sqp_alpha] < 1e-8
        && [sqp_norm_df] < [expr [sqp_eps]*abs([prg_f])]} {
	incr nullsteps 1
    } else {
	set nullsteps 0
    }
    if {$nullsteps > 5} {
	error stall
    }

    # update quasi-parallel running Tcl stuff
    catch {update}
  }

  puts $stream [format "\n%43d qp-it" $qp_iters]
  unset qp_iters

  return optimal
}
