/*
 * Prg_Di.C -- class definition
 *
 * rf, 1/29/95
 *
 * rf, 2/10/98
 *   - updated for HQP 1.3
 */

#include "Prg_Di.h"

#include <If_Bool.h>
#include <If_Float.h>

#include <If_Class.h>

IF_CLASS_DEFINE("Di", Prg_Di, Hqp_SqpProgram);

//--------------------------------------------------------------------------
Prg_Di::Prg_Di()
{
  _kmax = 60;
  _dt = 1.0 / (double)_kmax;
  _fscale = 1.0;
  _cns = true;

  horizon(0, _kmax);

  _ifList.append(new If_Bool("prg_cns", &_cns));
  _ifList.append(new If_Float("prg_fscale", &_fscale));
}

//--------------------------------------------------------------------------
Prg_Di::~Prg_Di()
{
}

//--------------------------------------------------------------------------
void Prg_Di::setup_vars(int k, VECP xmin, VECP xmax,
			VECP umin, VECP umax, VECP cmin, VECP cmax)
{
  alloc_vars(xmin, xmax, 2);
  if (k < _kmax)
    alloc_vars(umin, umax, 1);

  if (k == 0) {
    xmin[0] = xmax[0] = 1.0;
    xmin[1] = xmax[1] = 0.0;
  }
  else if (k < _kmax) {
    xmax[1] = 0.01;
  }
  else {
    xmin[0] = xmax[0] = -1.0;
    xmin[1] = xmax[1] = 0.0;
  }

  if (_cns) {
    if (k < _kmax) {
      alloc_vars(cmin, cmax, 1);
      cmax[0] = 0.01;
    }
  }
}

//--------------------------------------------------------------------------
void Prg_Di::setup_struct(int k, MATP fx, MATP fu, IVECP f_lin,
			  MATP cx, MATP cu, IVECP c_lin,
			  MATP Lxx, MATP Luu, MATP Lxu)
{
  if (k < _kmax) {
    m_zero(fx);
    m_zero(fu);

    fx[0][0] = 1.0;
    fx[1][0] = _dt;
    fx[1][1] = 1.0;

    fu[0][0] = _dt;
    fu[1][0] = 0.5*_dt*_dt;

    f_lin[0] = f_lin[1] = 1;

    if (_cns) {
      cx[0][0] = 0.5*_dt;
      cx[0][1] = 1.0;
      cu[0][0] = 0.0;
      c_lin[0] = 1;
    }
  }
}

//--------------------------------------------------------------------------
void Prg_Di::init_simulation(int k, VECP x, VECP u)
{
  if (k < _kmax)
    u[0] = -2.0;
}

//--------------------------------------------------------------------------
void Prg_Di::update_vals(int k, const VECP x, const VECP u,
			 VECP f, Real &f0, VECP c)
{
  if (k < _kmax) {
    f[0] = x[0] + u[0]*_dt;
    f[1] = x[0]*_dt + x[1] + u[0]*0.5*_dt*_dt;

    f0 = _fscale * u[0] * u[0] * _dt;

    if (_cns) {
      c[0] = x[1] + 0.5*_dt*x[0];
    }
  }
  else
    f0 = 0.0;
}

#ifdef GRDS
//--------------------------------------------------------------------------
void Prg_Di::update_grds(int k, const VECP x, const VECP u,
			 MATP fx, MATP fu, VECP f0x, VECP f0u,
			 MATP, MATP)
{
  v_zero(f0x);
  if (k < _kmax) {
    f0u[0] = _fscale * 2.0*u[0]*_dt;
  }
}
#endif

//==========================================================================
