/*
 * Prg_Di.h -- 
 *   - double integrator with state constraint
 *
 * rf, 1/29/95
 *
 * rf, 2/10/98
 *   - updated for HQP 1.3
 */

#ifndef Prg_Di_H
#define Prg_Di_H

#include "Hqp_Docp.h"

// uncomment this line to enable implementation of analytical gradients
// (otherwise finite difference approximation is used by default)
//#define GRDS 1

//--------------------------------------------------------------------------
class Prg_Di: public Hqp_Docp {

 protected:

  int	_kmax;
  double _dt;
  double _fscale;
  bool _cns;

  void setup_vars(int k,
		  VECP xmin, VECP xmax,
		  VECP umin, VECP umax,
		  VECP cmin, VECP cmax);
  void setup_struct(int k,
		    MATP fx, MATP fu, IVECP f_lin,
		    MATP cx, MATP cu, IVECP c_lin,
		    MATP Lxx, MATP Luu, MATP Lxu);
  void init_simulation(int k,
		       VECP x, VECP u);
  void update_vals(int k, const VECP x, const VECP u,
		   VECP f, Real &f0, VECP c);
#ifdef GRDS
  void update_grds(int k, const VECP x, const VECP u,
		   MATP fx, MATP fu, VECP f0x, VECP f0u,
		   MATP cx, MATP cu);
#endif
 public:

  Prg_Di();
  ~Prg_Di();

  char *name() {return "Di";}
};  

#endif


